@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
+# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
 #
 # Powered by the Bokeh Development Team.
 #
@@ -23,7 +23,7 @@ import pytest ; pytest
 
 # Bokeh imports
 from bokeh.document import Document
-from bokeh.util.testing import with_file_contents
+from bokeh._testing.util.filesystem import with_file_contents
 
 # Module under test
 import bokeh.application.handlers.script as bahs
@@ -36,8 +36,14 @@ import bokeh.application.handlers.script as bahs
 # General API
 #-----------------------------------------------------------------------------
 
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 class Test_ScriptHandler(object):
 
+    # Public methods ----------------------------------------------------------
+
     def test_runner_uses_source_from_filename(self):
         doc = Document()
         source = "# Test contents for script"
@@ -73,9 +79,9 @@ class Test_ScriptHandler(object):
             bahs.ScriptHandler()
 
 #-----------------------------------------------------------------------------
-# Dev API
+# Private API
 #-----------------------------------------------------------------------------
 
 #-----------------------------------------------------------------------------
-# Private API
+# Code
 #-----------------------------------------------------------------------------
