@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
+# Copyright (c) 2012 - 2018, Anaconda, Inc. All rights reserved.
 #
 # Powered by the Bokeh Development Team.
 #
@@ -17,8 +17,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -33,11 +31,14 @@ from bokeh.util.api import general, dev ; general, dev
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'Color',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-@general((1,0,0))
 class Color(object):
     ''' A base class for representing color objects.
 
@@ -47,7 +48,6 @@ class Color(object):
         return self.to_css()
 
     @staticmethod
-    @general((1,0,0))
     def clamp(value, maximum=None):
         ''' Clamp numeric values to be non-negative, an optionally, less than a
         given maximum.
@@ -71,7 +71,6 @@ class Color(object):
         else:
             return value
 
-    @general((1,0,0))
     def copy(self):
         ''' Copy this color.
 
@@ -80,7 +79,6 @@ class Color(object):
         '''
         raise NotImplementedError
 
-    @general((1,0,0))
     def darken(self, amount):
         ''' Darken (reduce the luminance) of this color.
 
@@ -98,7 +96,6 @@ class Color(object):
 
 
     @classmethod
-    @general((1,0,0))
     def from_hsl(cls, value):
         ''' Create a new color by converting from an HSL color.
 
@@ -115,7 +112,6 @@ class Color(object):
         raise NotImplementedError
 
     @classmethod
-    @general((1,0,0))
     def from_rgb(cls, value):
         ''' Create a new color by converting from an RGB color.
 
@@ -131,7 +127,6 @@ class Color(object):
         '''
         raise NotImplementedError
 
-    @general((1,0,0))
     def lighten(self, amount):
         ''' Lighten (increase the luminance) of this color.
 
@@ -147,7 +142,6 @@ class Color(object):
         hsl.l = self.clamp(hsl.l + amount, 1)
         return self.from_hsl(hsl)
 
-    @general((1,0,0))
     def to_css(self):
         ''' Return a CSS representation of this color.
 
@@ -160,7 +154,6 @@ class Color(object):
         raise NotImplementedError
 
 
-    @general((1,0,0))
     def to_hsl(self):
         ''' Create a new HSL color by converting from this color.
 
@@ -172,7 +165,6 @@ class Color(object):
         '''
         raise NotImplementedError
 
-    @general((1,0,0))
     def to_rgb(self):
         ''' Create a new HSL color by converting from this color.
 
