@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #-----------------------------------------------------------------------------
 # Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
 #
@@ -13,60 +14,37 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 import pytest ; pytest
 
-from bokeh.util.api import DEV, GENERAL ; DEV, GENERAL
-from bokeh.util.testing import verify_api ; verify_api
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
 # Standard library imports
 from mock import patch
-from tempfile import NamedTemporaryFile
-import os
 import re
 
 # External imports
 from PIL import Image
-import selenium.webdriver as webdriver
 
 # Bokeh imports
 from bokeh.models.plots import Plot
 from bokeh.models.ranges import Range1d
-from bokeh.io.export import terminate_web_driver
+from bokeh.io.webdriver import webdriver_control, terminate_webdriver
+from bokeh.plotting import figure
+from bokeh.resources import Resources
 
 # Module under test
 import bokeh.io.export as bie
 
-#-----------------------------------------------------------------------------
-# API Definition
-#-----------------------------------------------------------------------------
-
-api = {
-
-    GENERAL: (
-
-        ( 'export_png',  (1, 0, 0) ),
-        ( 'export_svgs', (1, 0, 0) ),
-
-    ), DEV: (
-
-        ( 'get_screenshot_as_png',      (1, 0, 0) ),
-        ( 'get_svgs',                   (1, 0, 0) ),
-        ( 'save_layout_html',           (1, 0, 0) ),
-        ( 'wait_until_render_complete', (1, 0, 0) ),
-        ( 'terminate_web_driver',       (1, 0, 0) ),
-
-    ),
-
-}
-
-Test_api = verify_api(bie, api)
-
 #-----------------------------------------------------------------------------
 # Setup
 #-----------------------------------------------------------------------------
 
+@pytest.fixture(scope='module')
+def webdriver():
+    driver = webdriver_control.create()
+    yield driver
+    terminate_webdriver(driver)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -90,18 +68,13 @@ def test_get_screenshot_as_png():
 
 @pytest.mark.unit
 @pytest.mark.selenium
-def test_get_screenshot_as_png_with_driver():
+def test_get_screenshot_as_png_with_driver(webdriver):
     layout = Plot(x_range=Range1d(), y_range=Range1d(),
                   plot_height=20, plot_width=20, toolbar_location=None,
                   outline_line_color=None, background_fill_color=None,
                   border_fill_color=None)
 
-    driver = webdriver.PhantomJS(service_log_path=os.path.devnull)
-
-    png = bie.get_screenshot_as_png(layout, driver=driver)
-
-    # Have to manually clean up the driver session
-    terminate_web_driver(driver)
+    png = bie.get_screenshot_as_png(layout, driver=webdriver)
 
     assert png.size == (20, 20)
     # a 20x20px image of transparent pixels
@@ -109,23 +82,33 @@ def test_get_screenshot_as_png_with_driver():
 
 @pytest.mark.unit
 @pytest.mark.selenium
-def test_get_screenshot_as_png_large_plot():
+def test_get_screenshot_as_png_large_plot(webdriver):
     layout = Plot(x_range=Range1d(), y_range=Range1d(),
                   plot_height=800, plot_width=800, toolbar_location=None,
                   outline_line_color=None, background_fill_color=None,
                   border_fill_color=None)
 
-    driver = webdriver.PhantomJS(service_log_path=os.path.devnull)
-    assert driver.get_window_size() == {'width': 400, 'height': 300}
-
-    bie.get_screenshot_as_png(layout, driver=driver)
+    bie.get_screenshot_as_png(layout, driver=webdriver)
 
     # LC: Although the window size doesn't match the plot dimensions (unclear
     # why), the window resize allows for the whole plot to be captured
-    assert driver.get_window_size() == {'width': 1366, 'height': 768}
+    assert webdriver.get_window_size() == {'width': 1366, 'height': 768}
 
-    # Have to manually clean up the driver session
-    terminate_web_driver(driver)
+@pytest.mark.unit
+@pytest.mark.selenium
+def test_get_screenshot_as_png_with_unicode_minified(webdriver):
+    p = figure(title="유니 코드 지원을위한 작은 테스트")
+
+    png = bie.get_screenshot_as_png(p, driver=webdriver, resources=Resources(mode="inline", minified=True))
+    assert len(png.tobytes()) > 0
+
+@pytest.mark.unit
+@pytest.mark.selenium
+def test_get_screenshot_as_png_with_unicode_unminified(webdriver):
+    p = figure(title="유니 코드 지원을위한 작은 테스트")
+
+    png = bie.get_screenshot_as_png(p, driver=webdriver, resources=Resources(mode="inline", minified=False))
+    assert len(png.tobytes()) > 0
 
 @pytest.mark.unit
 @pytest.mark.selenium
@@ -138,7 +121,7 @@ def test_get_svgs_no_svg_present():
 
 @pytest.mark.unit
 @pytest.mark.selenium
-def test_get_svgs_with_svg_present():
+def test_get_svgs_with_svg_present(webdriver):
 
     def fix_ids(svg):
         svg = re.sub(r'id="\w{12}"', 'id="X"', svg)
@@ -150,11 +133,8 @@ def test_get_svgs_with_svg_present():
                   outline_line_color=None, border_fill_color=None,
                   background_fill_color="red", output_backend="svg")
 
-    svg0 = fix_ids(bie.get_svgs(layout)[0])
-
-    driver = webdriver.PhantomJS(service_log_path=os.path.devnull)
-    svg1 = fix_ids(bie.get_svgs(layout)[0])
-    terminate_web_driver(driver) # Have to manually clean up the driver session
+    svg0 = fix_ids(bie.get_svgs(layout, driver=webdriver)[0])
+    svg1 = fix_ids(bie.get_svgs(layout, driver=webdriver)[0])
 
     svg2 = (
         '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" '
@@ -179,14 +159,13 @@ def test_get_svgs_with_svg_present():
     assert svg0 == svg2
     assert svg1 == svg2
 
-def test_save_layout_html_resets_plot_dims():
+def test_get_layout_html_resets_plot_dims():
     initial_height, initial_width = 200, 250
 
     layout = Plot(x_range=Range1d(), y_range=Range1d(),
                   plot_height=initial_height, plot_width=initial_width)
 
-    with NamedTemporaryFile() as tmp:
-        bie.save_layout_html(layout, tmp.name, height=100, width=100)
+    bie.get_layout_html(layout, height=100, width=100)
 
     assert layout.plot_height == initial_height
     assert layout.plot_width == initial_width
