@@ -5,13 +5,13 @@ logger = logging.getLogger(__name__)
 
 from six import string_types
 
-from ..core.properties import Any, Auto, Either, Enum, Int, Seq, Instance, String
-from ..core.enums import HorizontalLocation, VerticalLocation
+from ..core.properties import Any, Auto, Either, Enum, Int, List, Seq, Instance, String, Tuple
+from ..core.enums import HorizontalLocation, MarkerType, VerticalLocation
 from ..models import ColumnDataSource, Plot, Title, Tool, GraphRenderer
-from ..models import glyphs, markers
+from ..models import glyphs as _glyphs
+from ..models import markers as _markers
 from ..models.tools import Drag, Inspection, Scroll, Tap
 from ..util.options import Options
-from ..util.string import format_docstring
 from ..transform import linear_cmap
 from .helpers import (
     _get_range, _get_scale, _process_axis_and_grid, _process_tools_arg,
@@ -83,8 +83,19 @@ class FigureOptions(Options):
     The type of the y-axis.
     """)
 
+    tooltips = Either(String, List(Tuple(String, String)), help="""
+    An optional argument to configure tooltips for the Figure. This argument
+    accepts the same values as the ``HoverTool.tooltips`` property. If a hover
+    tool is specified in the ``tools`` argument, this value will override that
+    hover tools ``tooltips`` value. If no hover tool is specified in the
+    ``tools`` argument, then passing tooltips here will cause one to be created
+    and added.
+    """)
+
 class Figure(Plot):
-    ''' A subclass of :class:`~bokeh.models.plots.Plot` that simplifies plot
+    ''' Create a new Figure for plotting.
+
+    A subclass of :class:`~bokeh.models.plots.Plot` that simplifies plot
     creation with default axes, grids, tools, etc.
 
     Figure objects have many glyph methods that can be used to draw
@@ -93,7 +104,50 @@ class Figure(Plot):
     .. hlist::
         :columns: 3
 
-{glyph_methods}
+        * :func:`~bokeh.plotting.figure.Figure.annular_wedge`
+        * :func:`~bokeh.plotting.figure.Figure.annulus`
+        * :func:`~bokeh.plotting.figure.Figure.arc`
+        * :func:`~bokeh.plotting.figure.Figure.asterisk`
+        * :func:`~bokeh.plotting.figure.Figure.bezier`
+        * :func:`~bokeh.plotting.figure.Figure.circle`
+        * :func:`~bokeh.plotting.figure.Figure.circle_cross`
+        * :func:`~bokeh.plotting.figure.Figure.circle_x`
+        * :func:`~bokeh.plotting.figure.Figure.cross`
+        * :func:`~bokeh.plotting.figure.Figure.dash`
+        * :func:`~bokeh.plotting.figure.Figure.diamond`
+        * :func:`~bokeh.plotting.figure.Figure.diamond_cross`
+        * :func:`~bokeh.plotting.figure.Figure.ellipse`
+        * :func:`~bokeh.plotting.figure.Figure.hbar`
+        * :func:`~bokeh.plotting.figure.Figure.hex`
+        * :func:`~bokeh.plotting.figure.Figure.hex_tile`
+        * :func:`~bokeh.plotting.figure.Figure.image`
+        * :func:`~bokeh.plotting.figure.Figure.image_rgba`
+        * :func:`~bokeh.plotting.figure.Figure.image_url`
+        * :func:`~bokeh.plotting.figure.Figure.inverted_triangle`
+        * :func:`~bokeh.plotting.figure.Figure.line`
+        * :func:`~bokeh.plotting.figure.Figure.multi_line`
+        * :func:`~bokeh.plotting.figure.Figure.multi_polygons`
+        * :func:`~bokeh.plotting.figure.Figure.oval`
+        * :func:`~bokeh.plotting.figure.Figure.patch`
+        * :func:`~bokeh.plotting.figure.Figure.patches`
+        * :func:`~bokeh.plotting.figure.Figure.quad`
+        * :func:`~bokeh.plotting.figure.Figure.quadratic`
+        * :func:`~bokeh.plotting.figure.Figure.ray`
+        * :func:`~bokeh.plotting.figure.Figure.rect`
+        * :func:`~bokeh.plotting.figure.Figure.segment`
+        * :func:`~bokeh.plotting.figure.Figure.square`
+        * :func:`~bokeh.plotting.figure.Figure.square_cross`
+        * :func:`~bokeh.plotting.figure.Figure.square_x`
+        * :func:`~bokeh.plotting.figure.Figure.step`
+        * :func:`~bokeh.plotting.figure.Figure.text`
+        * :func:`~bokeh.plotting.figure.Figure.triangle`
+        * :func:`~bokeh.plotting.figure.Figure.vbar`
+        * :func:`~bokeh.plotting.figure.Figure.wedge`
+        * :func:`~bokeh.plotting.figure.Figure.x`
+
+    As well as a scatter function that can be parameterized by marker type:
+
+    * :func:`~bokeh.plotting.figure.Figure.scatter`
 
     There are also two specialized methods for stacking bars:
 
@@ -104,9 +158,8 @@ class Figure(Plot):
 
     * :func:`~bokeh.plotting.figure.Figure.hexbin`
 
-    In addition to all the Bokeh model property attributes documented below,
-    the ``Figure`` initializer also accepts the following options, which can
-    help simplify configuration:
+    In addition to all the ``Figure`` property attributes, the following
+    options are also accepted:
 
     .. bokeh-options:: FigureOptions
         :module: bokeh.plotting.figure
@@ -144,13 +197,13 @@ class Figure(Plot):
         _process_axis_and_grid(self, opts.x_axis_type, opts.x_axis_location, opts.x_minor_ticks, opts.x_axis_label, self.x_range, 0)
         _process_axis_and_grid(self, opts.y_axis_type, opts.y_axis_location, opts.y_minor_ticks, opts.y_axis_label, self.y_range, 1)
 
-        tool_objs, tool_map = _process_tools_arg(self, opts.tools)
+        tool_objs, tool_map = _process_tools_arg(self, opts.tools, opts.tooltips)
         self.add_tools(*tool_objs)
         _process_active_tools(self.toolbar, tool_map, opts.active_drag, opts.active_inspect, opts.active_scroll, opts.active_tap)
 
-    annular_wedge = _glyph_function(glyphs.AnnularWedge)
+    annular_wedge = _glyph_function(_glyphs.AnnularWedge)
 
-    annulus = _glyph_function(glyphs.Annulus, """
+    annulus = _glyph_function(_glyphs.Annulus, """
 Examples:
 
         .. bokeh-plot::
@@ -166,9 +219,9 @@ Examples:
 
     """)
 
-    arc = _glyph_function(glyphs.Arc)
+    arc = _glyph_function(_glyphs.Arc)
 
-    asterisk = _glyph_function(markers.Asterisk, """
+    asterisk = _glyph_function(_markers.Asterisk, """
 Examples:
 
     .. bokeh-plot::
@@ -183,9 +236,9 @@ Examples:
 
 """)
 
-    bezier = _glyph_function(glyphs.Bezier)
+    bezier = _glyph_function(_glyphs.Bezier)
 
-    circle = _glyph_function(markers.Circle, """
+    circle = _glyph_function(_markers.Circle, """
 .. note::
     Only one of ``size`` or ``radius`` should be provided. Note that ``radius``
     defaults to data units.
@@ -204,7 +257,7 @@ Examples:
 
 """)
 
-    circle_cross = _glyph_function(markers.CircleCross, """
+    circle_cross = _glyph_function(_markers.CircleCross, """
 Examples:
 
     .. bokeh-plot::
@@ -220,7 +273,7 @@ Examples:
 
 """)
 
-    circle_x = _glyph_function(markers.CircleX, """
+    circle_x = _glyph_function(_markers.CircleX, """
 Examples:
 
     .. bokeh-plot::
@@ -236,7 +289,7 @@ Examples:
 
 """)
 
-    cross = _glyph_function(markers.Cross, """
+    cross = _glyph_function(_markers.Cross, """
 Examples:
 
     .. bokeh-plot::
@@ -252,7 +305,23 @@ Examples:
 
 """)
 
-    diamond = _glyph_function(markers.Diamond, """
+    dash = _glyph_function(_markers.Dash, """
+Examples:
+
+    .. bokeh-plot::
+        :source-position: above
+
+        from bokeh.plotting import figure, output_file, show
+
+        plot = figure(plot_width=300, plot_height=300)
+        plot.dash(x=[1, 2, 3], y=[1, 2, 3], size=[10,20,25],
+                  color="#99D594", line_width=2)
+
+        show(plot)
+
+""")
+
+    diamond = _glyph_function(_markers.Diamond, """
 Examples:
 
     .. bokeh-plot::
@@ -268,7 +337,7 @@ Examples:
 
 """)
 
-    diamond_cross = _glyph_function(markers.DiamondCross, """
+    diamond_cross = _glyph_function(_markers.DiamondCross, """
 Examples:
 
     .. bokeh-plot::
@@ -284,7 +353,7 @@ Examples:
 
 """)
 
-    hbar = _glyph_function(glyphs.HBar, """
+    hbar = _glyph_function(_glyphs.HBar, """
 Examples:
 
     .. bokeh-plot::
@@ -298,7 +367,7 @@ Examples:
         show(plot)
 """)
 
-    ellipse = _glyph_function(glyphs.Ellipse, """
+    ellipse = _glyph_function(_glyphs.Ellipse, """
 Examples:
 
     .. bokeh-plot::
@@ -314,7 +383,7 @@ Examples:
 
 """)
 
-    hex = _glyph_function(markers.Hex, """
+    hex = _glyph_function(_markers.Hex, """
 Examples:
 
     .. bokeh-plot::
@@ -329,7 +398,7 @@ Examples:
 
 """)
 
-    hex_tile = _glyph_function(glyphs.HexTile, """
+    hex_tile = _glyph_function(_glyphs.HexTile, """
 Examples:
 
     .. bokeh-plot::
@@ -344,7 +413,7 @@ Examples:
 
 """)
 
-    image = _glyph_function(glyphs.Image, """
+    image = _glyph_function(_glyphs.Image, """
 .. note::
     If both ``palette`` and ``color_mapper`` are passed, a ``ValueError``
     exception will be raised. If neither is passed, then the ``Greys9``
@@ -352,16 +421,16 @@ Examples:
 
 """)
 
-    image_rgba = _glyph_function(glyphs.ImageRGBA, """
+    image_rgba = _glyph_function(_glyphs.ImageRGBA, """
 .. note::
     The ``image_rgba`` method accepts images as a two-dimensional array of RGBA
     values (encoded as 32-bit integers).
 
 """)
 
-    image_url = _glyph_function(glyphs.ImageURL)
+    image_url = _glyph_function(_glyphs.ImageURL)
 
-    inverted_triangle = _glyph_function(markers.InvertedTriangle, """
+    inverted_triangle = _glyph_function(_markers.InvertedTriangle, """
 Examples:
 
     .. bokeh-plot::
@@ -376,7 +445,7 @@ Examples:
 
 """)
 
-    line = _glyph_function(glyphs.Line, """
+    line = _glyph_function(_glyphs.Line, """
 Examples:
 
     .. bokeh-plot::
@@ -391,7 +460,7 @@ Examples:
 
 """)
 
-    multi_line = _glyph_function(glyphs.MultiLine, """
+    multi_line = _glyph_function(_glyphs.MultiLine, """
 .. note::
     For this glyph, the data is not simply an array of scalars, it is an
     "array of arrays".
@@ -411,7 +480,27 @@ Examples:
 
 """)
 
-    oval = _glyph_function(glyphs.Oval, """
+    multi_polygons = _glyph_function(_glyphs.MultiPolygons, """
+.. note::
+    For this glyph, the data is not simply an array of scalars, it is a
+    nested array.
+
+Examples:
+
+    .. bokeh-plot::
+       :source-position: above
+
+       from bokeh.plotting import figure, output_file, show
+
+       p = figure(plot_width=300, plot_height=300)
+       p.multi_polygons(xs=[[[[1, 1, 2, 2]]], [[[1, 1, 3], [1.5, 1.5, 2]]]],
+                        ys=[[[[4, 3, 3, 4]]], [[[1, 3, 1], [1.5, 2, 1.5]]]],
+                        color=['red', 'green'])
+       show(p)
+
+""")
+
+    oval = _glyph_function(_glyphs.Oval, """
 Examples:
 
     .. bokeh-plot::
@@ -427,7 +516,7 @@ Examples:
 
 """)
 
-    patch = _glyph_function(glyphs.Patch, """
+    patch = _glyph_function(_glyphs.Patch, """
 Examples:
 
     .. bokeh-plot::
@@ -442,7 +531,7 @@ Examples:
 
 """)
 
-    patches = _glyph_function(glyphs.Patches, """
+    patches = _glyph_function(_glyphs.Patches, """
 .. note::
     For this glyph, the data is not simply an array of scalars, it is an
     "array of arrays".
@@ -462,7 +551,7 @@ Examples:
 
 """)
 
-    quad = _glyph_function(glyphs.Quad, """
+    quad = _glyph_function(_glyphs.Quad, """
 Examples:
 
     .. bokeh-plot::
@@ -478,9 +567,9 @@ Examples:
 
 """)
 
-    quadratic = _glyph_function(glyphs.Quadratic)
+    quadratic = _glyph_function(_glyphs.Quadratic)
 
-    ray = _glyph_function(glyphs.Ray, """
+    ray = _glyph_function(_glyphs.Ray, """
 Examples:
 
     .. bokeh-plot::
@@ -496,7 +585,7 @@ Examples:
 
 """)
 
-    rect = _glyph_function(glyphs.Rect, """
+    rect = _glyph_function(_glyphs.Rect, """
 Examples:
 
     .. bokeh-plot::
@@ -512,7 +601,7 @@ Examples:
 
 """)
 
-    step = _glyph_function(glyphs.Step, """
+    step = _glyph_function(_glyphs.Step, """
 Examples:
 
     .. bokeh-plot::
@@ -528,7 +617,7 @@ Examples:
 """)
 
 
-    segment = _glyph_function(glyphs.Segment, """
+    segment = _glyph_function(_glyphs.Segment, """
 Examples:
 
     .. bokeh-plot::
@@ -545,7 +634,7 @@ Examples:
 
 """)
 
-    square = _glyph_function(markers.Square, """
+    square = _glyph_function(_markers.Square, """
 Examples:
 
     .. bokeh-plot::
@@ -560,7 +649,7 @@ Examples:
 
 """)
 
-    square_cross = _glyph_function(markers.SquareCross, """
+    square_cross = _glyph_function(_markers.SquareCross, """
 Examples:
 
     .. bokeh-plot::
@@ -576,7 +665,7 @@ Examples:
 
 """)
 
-    square_x = _glyph_function(markers.SquareX, """
+    square_x = _glyph_function(_markers.SquareX, """
 Examples:
 
     .. bokeh-plot::
@@ -592,7 +681,7 @@ Examples:
 
 """)
 
-    text = _glyph_function(glyphs.Text, """
+    text = _glyph_function(_glyphs.Text, """
 .. note::
     The location and angle of the text relative to the ``x``, ``y`` coordinates
     is indicated by the alignment and baseline text properties.
@@ -602,7 +691,7 @@ Returns:
 
 """)
 
-    triangle = _glyph_function(markers.Triangle, """
+    triangle = _glyph_function(_markers.Triangle, """
 Examples:
 
     .. bokeh-plot::
@@ -618,7 +707,7 @@ Examples:
 
 """)
 
-    vbar = _glyph_function(glyphs.VBar, """
+    vbar = _glyph_function(_glyphs.VBar, """
 Examples:
 
     .. bokeh-plot::
@@ -633,7 +722,7 @@ Examples:
 
 """)
 
-    wedge = _glyph_function(glyphs.Wedge, """
+    wedge = _glyph_function(_glyphs.Wedge, """
 Examples:
 
     .. bokeh-plot::
@@ -649,7 +738,7 @@ Examples:
 
 """)
 
-    x = _glyph_function(markers.X, """
+    x = _glyph_function(_markers.X, """
 Examples:
 
     .. bokeh-plot::
@@ -666,48 +755,58 @@ Examples:
 
     # -------------------------------------------------------------------------
 
+    _scatter = _glyph_function(_markers.Scatter)
+
     def scatter(self, *args, **kwargs):
-        """ Creates a scatter plot of the given x and y items.
+        ''' Creates a scatter plot of the given x and y items.
 
         Args:
             x (str or seq[float]) : values or field names of center x coordinates
+
             y (str or seq[float]) : values or field names of center y coordinates
+
             size (str or list[float]) : values or field names of sizes in screen units
-            marker (str, optional): a valid marker_type, defaults to "circle"
+
+            marker (str, or list[str]): values or field names of marker types
+
             color (color value, optional): shorthand to set both fill and line color
+
             source (:class:`~bokeh.models.sources.ColumnDataSource`) : a user-supplied data source.
                 An attempt will be made to convert the object to :class:`~bokeh.models.sources.ColumnDataSource`
                 if needed. If none is supplied, one is created for the user automatically.
+
             **kwargs: :ref:`userguide_styling_line_properties` and :ref:`userguide_styling_fill_properties`
 
         Examples:
 
-            >>> p.scatter([1,2,3],[4,5,6], fill_color="red")
-            >>> p.scatter("data1", "data2", source=data_source, ...)
+            >>> p.scatter([1,2,3],[4,5,6], marker="square", fill_color="red")
+            >>> p.scatter("data1", "data2", marker="mtype", source=data_source, ...)
 
-        """
-        markertype = kwargs.pop("marker", "circle")
+        .. note::
+            When passing ``marker="circle"`` it is also possible to supply a
+            ``radius`` value in data-space units. When configuring marker type
+            from a data source column, *all* markers incuding circles may only
+            be configured with ``size`` in screen units.
 
-        if markertype not in _marker_types:
-            raise ValueError("Invalid marker type '%s'. Use markers() to see a list of valid marker types." % markertype)
+        '''
+        marker_type = kwargs.pop("marker", "circle")
 
-        # TODO (bev) make better when plotting.scatter is removed
-        conversions = {
-            "*": "asterisk",
-            "+": "cross",
-            "o": "circle",
-            "ox": "circle_x",
-            "o+": "circle_cross"
-        }
-        if markertype in conversions:
-            markertype = conversions[markertype]
+        if isinstance(marker_type, string_types) and marker_type in _MARKER_SHORTCUTS:
+            marker_type = _MARKER_SHORTCUTS[marker_type]
 
-        return getattr(self, markertype)(*args, **kwargs)
+        # The original scatter implementation allowed circle scatters to set a
+        # radius. We will leave this here for compatibility but note that it
+        # only works when the marker type is "circle" (and not referencing a
+        # data source column). Consider deprecating in the future.
+        if marker_type == "circle" and "radius" in kwargs:
+            return self.circle(*args, **kwargs)
+        else:
+            return self._scatter(*args, marker=marker_type, **kwargs)
 
     def hexbin(self, x, y, size, orientation="pointytop", palette="Viridis256", line_color=None, fill_color=None, aspect_scale=1, **kwargs):
         ''' Perform a simple equal-weight hexagonal binning.
 
-        A :class:`~bokeh.models.glyphs.HexTile` glyph will be added to display
+        A :class:`~bokeh.models._glyphs.HexTile` glyph will be added to display
         the binning. The :class:`~bokeh.models.sources.ColumnDataSource` for
         the glyph will have columns ``q``, ``r``, and ``count``, where ``q``
         and ``r`` are `axial coordinates`_ for a tile, and ``count`` is the
@@ -822,6 +921,10 @@ Examples:
             stackers (seq[str]) : a list of data source field names to stack
                 successively for ``left`` and ``right`` bar coordinates.
 
+                Additionally, the ``name`` of the renderer will be set to
+                the value of each successive stacker (this is useful with the
+                special hover variable ``$name``)
+
         Any additional keyword arguments are passed to each call to ``hbar``.
         If a keyword value is a list or tuple, then each call will get one
         value from the sequence.
@@ -843,8 +946,8 @@ Examples:
 
             .. code-block:: python
 
-                p.hbar(bottom=stack(),       top=stack('2016'),         x=10, width=0.9, color='blue', source=source)
-                p.hbar(bottom=stack('2016'), top=stack('2016', '2017'), x=10, width=0.9, color='red', source=source)
+                p.hbar(bottom=stack(),       top=stack('2016'),         x=10, width=0.9, color='blue', source=source, name='2016')
+                p.hbar(bottom=stack('2016'), top=stack('2016', '2017'), x=10, width=0.9, color='red',  source=source, name='2017')
 
         '''
         result = []
@@ -860,6 +963,10 @@ Examples:
             stackers (seq[str]) : a list of data source field names to stack
                 successively for ``left`` and ``right`` bar coordinates.
 
+                Additionally, the ``name`` of the renderer will be set to
+                the value of each successive stacker (this is useful with the
+                special hover variable ``$name``)
+
         Any additional keyword arguments are passed to each call to ``vbar``.
         If a keyword value is a list or tuple, then each call will get one
         value from the sequence.
@@ -881,8 +988,8 @@ Examples:
 
             .. code-block:: python
 
-                p.vbar(bottom=stack(),       top=stack('2016'),         x=10, width=0.9, color='blue', source=source)
-                p.vbar(bottom=stack('2016'), top=stack('2016', '2017'), x=10, width=0.9, color='red', source=source)
+                p.vbar(bottom=stack(),       top=stack('2016'),         x=10, width=0.9, color='blue', source=source, name='2016')
+                p.vbar(bottom=stack('2016'), top=stack('2016', '2017'), x=10, width=0.9, color='red',  source=source, name='2017')
 
 
         '''
@@ -913,74 +1020,29 @@ Examples:
         return graph_renderer
 
 def figure(**kwargs):
-    ''' Create a new :class:`~bokeh.plotting.figure.Figure` for plotting.
-
-    Figure objects have many glyph methods that can be used to draw
-    vectorized graphical glyphs:
-
-    .. hlist::
-        :columns: 3
-
-{glyph_methods}
-
-    There are also two specialized methods for stacking bars:
-
-    * :func:`~bokeh.plotting.figure.Figure.hbar_stack`
-    * :func:`~bokeh.plotting.figure.Figure.vbar_stack`
-
-    And one specialized method for making simple hexbin plots:
-
-    * :func:`~bokeh.plotting.figure.Figure.hexbin`
-
-    In addition to the standard :class:`~bokeh.plotting.figure.Figure`
-    property values (e.g. ``plot_width`` or ``sizing_mode``) the following
-    additional options can be passed as well:
-
-    .. bokeh-options:: FigureOptions
-        :module: bokeh.plotting.figure
-
-    Returns:
-       Figure
-
-    '''
-
     return Figure(**kwargs)
+figure.__doc__ = Figure.__doc__
 
-
-_marker_types = [
-    "asterisk",
-    "circle",
-    "circle_cross",
-    "circle_x",
-    "cross",
-    "diamond",
-    "diamond_cross",
-    "hex",
-    "inverted_triangle",
-    "square",
-    "square_x",
-    "square_cross",
-    "triangle",
-    "x",
-    "*",
-    "+",
-    "o",
-    "ox",
-    "o+",
-]
+_MARKER_SHORTCUTS = {
+    "*"  : "asterisk",
+    "+"  : "cross",
+    "o"  : "circle",
+    "ox" : "circle_x",
+    "o+" : "circle_cross",
+    "-"  : "dash",
+    "v"  : "inverted_triangle",
+    "^"  : "triangle",
+}
 
 def markers():
-    """ Prints a list of valid marker types for scatter()
+    ''' Prints a list of valid marker types for scatter()
 
     Returns:
         None
-    """
-    print("Available markers: \n - " + "\n - ".join(_marker_types))
+    '''
+    print("Available markers: \n\n - " + "\n - ".join(list(MarkerType)))
+    print()
+    print("Shortcuts: \n\n" + "\n".join(" %r: %s" % item for item in _MARKER_SHORTCUTS.items()))
 
 _color_fields = set(["color", "fill_color", "line_color"])
 _alpha_fields = set(["alpha", "fill_alpha", "line_alpha"])
-
-_gms = sorted(x for x in dir(Figure) if getattr(getattr(Figure, x), 'glyph_method', False))
-_gms = "\n".join("        * :func:`~bokeh.plotting.figure.Figure.%s`" % x for x in _gms)
-Figure.__doc__ = format_docstring(Figure.__doc__, glyph_methods=_gms)
-figure.__doc__ = format_docstring(figure.__doc__, glyph_methods=_gms)
