@@ -8,7 +8,7 @@ log = logging.getLogger(__name__)
 
 from tornado import gen
 
-from bokeh.embed import server_html_page_for_session
+from bokeh.embed.server import server_html_page_for_session
 
 from .session_handler import SessionHandler
 
@@ -16,17 +16,12 @@ class DocHandler(SessionHandler):
     ''' Implements a custom Tornado handler for document display page
 
     '''
-    def __init__(self, tornado_app, *args, **kw):
-        super(DocHandler, self).__init__(tornado_app, *args, **kw)
-
-    def initialize(self, *args, **kw):
-        pass
-
     @gen.coroutine
     def get(self, *args, **kwargs):
         session = yield self.get_session()
 
-        page = server_html_page_for_session(session.id, self.application.resources(),
+        page = server_html_page_for_session(session,
+                                            resources=self.application.resources(),
                                             title=session.document.title,
                                             template=session.document.template,
                                             template_variables=session.document.template_variables)
